package jalview.rest;

import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jalview.api.AlignmentViewPanel;
import jalview.bin.Console;
import jalview.datamodel.AlignmentI;
import jalview.datamodel.SequenceI;
import jalview.gui.AlignFrame;

public class GetCrossReferencesEndpoint extends AbstractEndpointAsync
{
  public GetCrossReferencesEndpoint(API api)
  {
    super(api, path, name, parameters, description);
  }

  private static final String path = "getcrossreferences";

  private static final String name = "Get Cross References";

  private static final String parameters = "<resource name>/<sequence ids>?fromId=<id>";

  private static final String description = "Get Cross References for alignment <id>";

  private String resourceName;

  private AlignFrame af;

  private SequenceI[] seqs;

  @Override
  protected void initialise(HttpServletRequest request,
          HttpServletResponse response)
  {
    if (!checkParameters(request, response, 1))
    {
      return;
    }
    String[] parameters = getEndpointPathParameters(request);

    resourceName = parameters[0];

    String fromId = request.getParameter("fromId");
    setId(request, resourceName + "::" + fromId);

    if (checkStatus(request, response))
    {
      return;
    }

    af = getAlignFrameFromId(request);
    if (af == null)
    {
      returnError(request, response, "Could not find results");
      return;
    }

    if (af.canShowProducts())
    {
      List<String> products = af.getProducts();
      if (products == null)
      {
        returnError(request, response,
                "no cross reference products available");
        return;
      }
      if (!products.contains(resourceName))
      {
        StringBuilder sb = new StringBuilder();
        sb.append("cross reference product '").append(resourceName)
                .append("' not available: available products are\n");
        sb.append("products=");
        boolean first = true;
        for (String p : products)
        {
          if (!first)
          {
            sb.append(",");
            first = false;
          }
          sb.append(p);
        }
        sb.append("\n");
        returnError(request, response, sb.toString());
        return;
      }
    }

    seqs = af.getViewport().getAlignment().getSequencesArray();
    if (seqs == null || seqs.length == 0)
    {
      // nothing to do
      returnError(request, response, "no sequences selected");
      return;
    }

    final boolean dna = af.getViewport().getAlignment().isNucleotide();
    setCompletableFuture(
            af.showProductsFor(seqs, dna, resourceName, true, getId()));
  }

  protected void processAsync(HttpServletRequest request,
          HttpServletResponse response, Map<String, Object> map)
  {
    // all the work being done by the AlignFrame!
    Console.warn("THIS SHOULD NOT BE RUN");
  }

  protected String finishedResponseString(HttpServletRequest request,
          HttpServletResponse response)
  {
    AlignFrame af = AlignFrame.getAlignFrameFromRestId(getId());
    if (af == null)
    {
      return null;
    }
    List<AlignmentViewPanel> aps = (List<AlignmentViewPanel>) af
            .getAlignPanels();
    StringBuilder sb = new StringBuilder();
    for (AlignmentViewPanel ap : aps)
    {
      AlignmentI al = ap.getAlignment();
      List<SequenceI> seqs = (List<SequenceI>) al.getSequences();
      for (SequenceI seq : seqs)
      {
        if (sb.length() > 0)
          sb.append(",");
        sb.append(seq.getName());
      }
    }
    sb.insert(0, "sequences=");
    sb.append("\n");
    return sb.toString();
  }

}
