package jalview.rest;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jalview.api.AlignmentViewPanel;
import jalview.datamodel.AlignmentI;
import jalview.datamodel.SequenceI;
import jalview.gui.AlignFrame;
import jalview.structure.StructureSelectionManager;

public class HighlightSequenceEndpoint extends AbstractEndpoint
{
  public HighlightSequenceEndpoint(API api)
  {
    super(api, path, name, parameters, description);
  }

  protected static final String path = "highlightsequence";

  private static final String name = "Highlight sequence position";

  private static final String parameters = "<sequence names>/<position>";

  private static final String description = "Highlight the specified sequences at the specified position";

  public void processEndpoint(HttpServletRequest request,
          HttpServletResponse response)
  {
    if (!checkParameters(request, response, 2))
    {
      return;
    }
    String[] parameters = getEndpointPathParameters(request);

    String posString = parameters[1];
    int pos = -1;
    try
    {
      pos = Integer.parseInt(posString);
    } catch (NumberFormatException e)
    {
      returnError(request, response,
              "Could not parse postition integer " + posString);
    }

    String sequenceNames = parameters[0];

    Map<SequenceI, StructureSelectionManager> ssmMap = new HashMap<>();
    AlignFrame[] alignFrames = getAlignFrames(request, true);
    if (alignFrames == null)
    {
      returnError(request, response, "could not find results");
      return;
    }
    for (int i = 0; i < alignFrames.length; i++)
    {
      AlignFrame af = alignFrames[i];
      List<AlignmentViewPanel> aps = (List<AlignmentViewPanel>) af
              .getAlignPanels();
      for (AlignmentViewPanel ap : aps)
      {
        StructureSelectionManager ssm = ap.getStructureSelectionManager();
        AlignmentI al = ap.getAlignment();
        List<SequenceI> seqs = (List<SequenceI>) al.getSequences();
        for (SequenceI seq : seqs)
        {
          if (sequenceNames.equals(seq.getName()))
          {
            ssmMap.put(seq, ssm);
          }
        }
      }
    }
    // highlight
    for (SequenceI seq : ssmMap.keySet())
    {
      StructureSelectionManager ssm = ssmMap.get(seq);
      if (ssm == null)
      {
        continue;
      }
      ssm.mouseOverSequence(seq, pos, -1, null);
    }

  }
}